/* lorien.c 

 Routines for parsing the command line and getting things rolling.

 Chris Eleveld      (The Insane Hermit)
 Robert Slaven      (The Celestial Knight/Tigger)
 Jerry Bolton, Jr.  (Creel)
 Dave Mott          (Energizer Rabbit)

*/

#define _LORIEN_C_

#include <stdio.h>
#include <sys/types.h>
#ifdef POSIX
#  include <sys/select.h>
#endif
#include <sys/time.h>
#include <sys/resource.h>
#include "lorien.h"
#include <ctype.h>
#include <signal.h>
#include <unistd.h>

int port;

int 
main(argc, argv)
    int argc;
    char **argv;
{
    printf("MAXCONN = %d\n",MAXCONN);
    get_timestamp();
    if (argc == 1) {
	fprintf(stderr, "USAGE: %s [-d] portnum\n", *argv);
	exit(1);
    }

    handleargs(argc, argv+1);

    ban_read_blockfile();
    init_read_powerfile();

    doit(port);

    return 0;
}

int
handleargs(argc, argv)
    int argc;
    char **argv;
{
    int fdaemon = 0;
    int childid;
    char *logfile = (char *)0;

    while (--argc) {
	if (**argv == '-') {
	    switch ((*argv)[1]) {
            case 'd':
		fdaemon = 1;
		break;
            case 'l':
		if ((*argv)[2] != '\000') {
		    logfile = (*argv)+2;
		} else {
		    if (argc < 1) {
			fprintf(stderr, USAGE);
			fprintf(stderr, "-l requires you to specify a file name.\n");
			exit(1);
		    }
		    argc--, argv++;
		    logfile = *argv;
		}
		break;
            default:
		fprintf(stderr, "lorien: Unrecognized command line option <%s> on command line.\n", *argv); 
		exit(3);
	    }
	} else if (isdigit(**argv)) {
	    port = atoi(*argv);
	} else {
	    fprintf(stderr, "lorien: Unrecognized word <%s> on command line .\n", *argv);
	    exit(3);
	}
	
	argv++;
    }

    if (logfile != (char *)0)
	if (freopen(logfile, "w", stderr) == NULL) {
	    printf("lorien: unable to open logfile, %s, on stderr.\n",
		   logfile);
	    exit(4);
	}
    if (fdaemon)  {
	switch(childid = fork()) {
	case -1:
            fprintf(stdout, "ERROR starting daemon. exitting.\n");
            perror("lorien: fork");
            exit(2);
            break;
	case 0:
            (void) fclose(stdin);
            (void) fclose(stdout);
            close(0);
            close(1);
            if (logfile == (char *)0)
		(void) fclose(stderr);
            break;
	default:
            printf("Daemon succesfully Started on port %d.\n", port);
            printf("Daemon's pid is %d.\n", childid);
            printf("This is the parent process signing off.\n");
            fclose(stdin);
            fclose(stdout);
            close(0);
            close(1);
            exit(0);
	}
	    /* This line generates a warning if your header files are broken
	       and don't properly cast SIG_IGN. */
	signal(SIGPIPE,SIG_IGN); 
    }
    else {
	fclose(stdin);
	fclose(stdout);
	close(0);
	close(1);
    }
}

int
die(exitstat)
    int exitstat;
{
    exit(exitstat);
}
